----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:15:36 01/26/2010 
-- Design Name: 
-- Module Name:    Audio32_PISO - BEHAVE 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY	Audio32_PISO is
	PORT(
				DIN		:IN	STD_LOGIC_VECTOR (31 DOWNTO 0);	-- Parallel audio data in
				DHOLD	:IN	STD_LOGIC;											-- Used to hold audio channel data to shifter
				SI			:IN	STD_LOGIC;											-- Shifter fill value, default tied to gnd
				CLK		:IN	STD_LOGIC;											--
				RST		:IN	STD_LOGIC;											--
				RSYNC	:IN	STD_LOGIC;											-- Used to sync up shifter load for serial data out
				COUNT	:OUT	STD_LOGIC_VECTOR(4 downto 0);			--	Counter for shift LD signal
				LOAD	:OUT	STD_LOGIC;											-- Monitor the shift LD signal
				SO		:OUT	STD_LOGIC											--	Serial audio data out
			);
END	Audio32_PISO;

ARCHITECTURE	BEHAVE of Audio32_PISO is

SIGNAL PDAT	:	STD_LOGIC_VECTOR	(31 DOWNTO 0);			-- PARALLEL DATA
SIGNAL	TEMP	:	STD_LOGIC_VECTOR	(31 DOWNTO 0);			-- SHIFT REG INPUT
SIGNAL	CNTR	:	STD_LOGIC_VECTOR	( 4 DOWNTO 0);			-- COUNTER FOR LOAD/HOLD
SIGNAL	SLOAD	:	STD_LOGIC;													-- SHIFT LOAD ENABLE
SIGNAL S_IN		:	STD_LOGIC;													--	SHIFT IN VALUE

BEGIN
--=================================================================================================================================
-- HOLD AUDIO DATA REGISTER, 32-BIT
--=================================================================================================================================
HOLD_REGISTER:PROCESS(CLK, RST, DIN, DHOLD)
	BEGIN
	IF RISING_EDGE (CLK) THEN
		IF (RST = '1') THEN
			PDAT	<= (OTHERS => '0');
		ELSIF(DHOLD = '1')THEN
			PDAT	<=	DIN;
		ELSE
			PDAT	<=	PDAT;
		END IF;
	END IF;
END PROCESS;
--=================================================================================================================================
-- SHIFT AUDIO DATA REGISTER, 32-BIT
--=================================================================================================================================
PROCESS(CLK, RSYNC, PDAT)
	BEGIN
	IF RISING_EDGE (CLK) THEN
	S_IN	<=	SI;
		IF (RSYNC = '1') THEN
			CNTR	<= (OTHERS => '1');
				IF (CNTR = "11111") THEN -- MAY NEED TO SET TO "11111"
					SLOAD	<= '1';
				ELSE
					SLOAD	<= '0';
				END IF;
		ELSE
			SLOAD	<= '0';
			CNTR	<= CNTR + '1';
				IF (CNTR = "11111") THEN -- MAY NEED TO SET TO "11111"
					SLOAD	<= '1';
				ELSIF(CNTR = "00000") THEN
					TEMP	<= PDAT;
					SLOAD	<= '0';
				ELSE
					TEMP <= TEMP(30 downto 0)& S_IN;
				END IF;
		END IF;
	END IF;
END PROCESS;

SO		<=	TEMP(31);
COUNT	<=	CNTR;
LOAD	<=	SLOAD;

END	BEHAVE;

