-- Generic Terminal counter
-- Dec 22, 2004
-- Anthony Medina
-- Made this a sync reset
--
LIBRARY IEEE ;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY BPCLK_MONITOR IS
	GENERIC(WIDTH	: INTEGER	:= 24);
PORT(
		BPCOUNT		:IN		STD_LOGIC_VECTOR(WIDTH - 1 DOWNTO 0);
		CLK_D		:IN		STD_LOGIC;	--	DANTE MCLK USED AS REF
		RST			:IN		STD_LOGIC;
		INACTIVE	:OUT	STD_LOGIC	--	INDICATES BP CLK DEAD
	);
END BPCLK_MONITOR;

ARCHITECTURE Behave OF BPCLK_MONITOR IS

SIGNAL	QTMP		:	STD_LOGIC_VECTOR (WIDTH-1 DOWNTO 0)	:=	(OTHERS => '0');
SIGNAL	SMPLCNT1	:	STD_LOGIC_VECTOR (WIDTH-1 DOWNTO 0)	:=	(OTHERS => '0');
SIGNAL	ACTIVTY		:	STD_LOGIC;


BEGIN
PROCESS(CLK_D,RST,QTMP,BPCOUNT)
--
--	Use 3 for simulation.
--	1,440,000	Dec	=>	15F900	Hex:	30	seconds,	16#15F900#
--	960,000		Dec	=>	0EA600	Hex:	20	seconds,	16#0EA600#
--	480,000		Dec	=>	075300	Hex:	10	seconds,	16#075300#
--	144,000		Dec	=>	23280	Hex:	3	seconds,	16#23280#
--	 48,000		Dec	=>	BB80	Hex:	1	second,		16#BB80#
--
--	Try to keep this as minimal as possible but enough to allow the Dante
--	to boot up and switch over SCLK from base 3MHz to 24MHz.
--	This takes approximately 3 to 7 seconds
--
--	The following values are for 50MHz (20nS) clock cycle. If used for Dante
--	SCLK/MCLK then values would be double
--
--	120mS @ 50MHz would be 240mS at 25MHz etc.
--
--	CONSTANT	TIMER			:	INTEGER	:=	16#15F900#;	--	30	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#0EA600#;	--	DEFAULT 20 seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#075300#;	--	10	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#23280#;	--	3	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#BB80#;	--	1	second
--	CONSTANT	TIMER			:	INTEGER	:=	16#8340#;	--	700 mS
--	CONSTANT	TIMER			:	INTEGER	:=	16#5A00#;	--	480 mS
--	CONSTANT	TIMER			:	INTEGER	:=	16#2D00#;	--	240 mS
--	CONSTANT	TIMER			:	INTEGER	:=	16#1680#;	--	120 mS
	CONSTANT	TIMER			:	INTEGER	:=	16#0020#;	--	Very short
--	CONSTANT	TIMER			:	INTEGER	:=	16#000003#;	--	Use this for simulation only
--
--
BEGIN

IF(RST = '1') THEN
	QTMP		<=	CONV_STD_LOGIC_VECTOR(0,WIDTH);
	SMPLCNT1	<=	(OTHERS => '0');
ELSIF RISING_EDGE(CLK_D) THEN
	QTMP	<=	QTMP + '1';
	IF(QTMP(5 DOWNTO 0) = CONV_STD_LOGIC_VECTOR(TIMER,6)) THEN
		SMPLCNT1	<=	BPCOUNT;				
	END IF;
END IF;
END PROCESS;

CHECK_IF_EQUAL : PROCESS(CLK_D,SMPLCNT1,BPCOUNT,QTMP) IS
	CONSTANT	TIMER2	:	INTEGER	:=	16#0030#;	--	Very short
BEGIN
  -- Put sequential statements here.
	IF RST = '1' THEN
		ACTIVTY		<=	'0';
	ELSIF(QTMP(5 DOWNTO 0) = CONV_STD_LOGIC_VECTOR(TIMER2,6)) THEN
		IF(SMPLCNT1 = BPCOUNT) THEN
			ACTIVTY	<=	'1';
		ELSE
			ACTIVTY	<=	'0';
		END IF;
	END IF;

	INACTIVE	<=	ACTIVTY;

END PROCESS CHECK_IF_EQUAL;

END Behave;
