--===================================================================================================
--	Company			: 
--	Engineer			: 
--
--	Create Date		:	12:29:33 02/26/2010 
--	Design Name		:	BPI2S MEMSTOR TO DANTE TX FIFO TRANSFER
--	Module Name		:	BPI2S_MEMXFER - BEHAVE 
--	Project Name		:	KRL DANTE
--	Target Devices	:	SPARTAN-3AN
--	Tool versions		:	ISE 9.2
--	Description		: 
--
--	Dependencies	: 
--
--	Revision			: 
--	Revision 0.01 - File Created
--	Additional Comments: 
--
--================================================================================================
LIBRARY	IEEE;
USE	IEEE.STD_LOGIC_1164.ALL;
USE	IEEE.STD_LOGIC_ARITH.ALL;
USE	IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY	BPI2S_MEMXFER is
	PORT(
				MSTOR_CYC_DONE	:IN	STD_LOGIC;
				BPMCLK					:IN	STD_LOGIC;
				RST							:IN	STD_LOGIC;
				BPI2S_MRD				:OUT	STD_LOGIC;
				BPI2S_MRD_ADDR		:OUT	STD_LOGIC_VECTOR (4 DOWNTO 0);
				BPI2S_FIFO_WR		:OUT	STD_LOGIC;
				STATE						:OUT	STD_LOGIC_VECTOR	(1 DOWNTO 0)
			);
END	BPI2S_MEMXFER;

ARCHITECTURE	BEHAVE of BPI2S_MEMXFER is

TYPE state_type is (S0, S1, S2);

SIGNAL CURRENT_STATE, NEXT_STATE			:	state_type;										--	STATE transitions
SIGNAL STATE_XFER										:	STD_LOGIC_VECTOR (1 DOWNTO 0);	--	Reflect current state
SIGNAL I2SRD_ADDR									:	STD_LOGIC_VECTOR (5 DOWNTO 0);	--	I2S MEM RD ADDR
SIGNAL	RDXFER, MRD, WRFIFO						:	STD_LOGIC;										--	RD/WR TRANSFER CONTROL SIGNALS

BEGIN
--===================================================================================================
-- CLOCKED PROCESS
--===================================================================================================
REGISTERS: PROCESS(BPMCLK, RST, RDXFER)										--
	VARIABLE	ADDRCNT	: INTEGER	RANGE 32 DOWNTO 0	:=	0;			--	COUNTER FOR MEM READ ADDRESS
BEGIN																									--
																											--
	IF RST = '1' THEN																				--
		CURRENT_STATE	<=	S0;															--	Async reset to RSYNC state
		STATE					<=	"00";															--	STATE STARTS AT ZERO
		ADDRCNT				:=	0;																--	SET MEM WRITE ADDRESS COUNT TO ZERO	
	ELSIF RISING_EDGE(BPMCLK) THEN													--
		CURRENT_STATE	<=	NEXT_STATE;											--
		STATE					<=	STATE_XFER;											--	Output reflects state
		IF (RDXFER	=	'1') THEN																--	EITHER RD OR WR TRANSFER IS ENABLED
			MRD			<=	NOT MRD;															--
			IF(MRD	=	'1')THEN																	--
				ADDRCNT	:=	ADDRCNT	+	1;												--
				WRFIFO		<=	NOT WRFIFO;
			ELSE																							--
				ADDRCNT	:=	ADDRCNT;													--
				WRFIFO		<=	'0';
			END IF;																						--
		ELSE																								--
			ADDRCNT	:=	0;																		--
			MRD			<=	'0';																	--
			WRFIFO		<=	'0';
		END IF;																							--
		I2SRD_ADDR	<=	CONV_STD_LOGIC_VECTOR(ADDRCNT,6);		--	
																											--
--	Physical outputs																					--
		BPI2S_MRD					<=	MRD;												--	
		BPI2S_MRD_ADDR(0)	<=	I2SRD_ADDR(0);								--	
		BPI2S_MRD_ADDR(1)	<=	I2SRD_ADDR(1);								--	
		BPI2S_MRD_ADDR(2)	<=	I2SRD_ADDR(2);								--	
		BPI2S_MRD_ADDR(3)	<=	I2SRD_ADDR(3);								--	
		BPI2S_MRD_ADDR(4)	<=	I2SRD_ADDR(4);								--
		BPI2S_FIFO_WR			<=	WRFIFO;											--
	END IF;
END PROCESS REGISTERS;
--===================================================================================================
-- ASYNC PROCESS
--===================================================================================================
TRANSITIONS: PROCESS(CURRENT_STATE, MSTOR_CYC_DONE, I2SRD_ADDR)
BEGIN
	CASE CURRENT_STATE is
--===================================================================================================
--===================================================================================================
		WHEN S0	=>							-- STATE 0, INITIAL START CHECKS FOR START OF MEM TRANSFER OF I2S DATA FROM BACKPLANE
			IF ( MSTOR_CYC_DONE	=	'1' ) THEN
				RDXFER		<=	'0';
				NEXT_STATE	<=	S1;
				STATE_XFER	<=	"01";
			ELSE
				RDXFER		<=	'0';
				NEXT_STATE	<=	S0;
				STATE_XFER	<=	"00";
			END IF;
--===================================================================================================
		WHEN S1	=>							-- STATE 1, WAIT UNTIL THE WRITE TRANSFERS ARE COMPLETE BEFORE TRANSFERRING TO THE DANTE TX FIFO
			IF ( MSTOR_CYC_DONE	=	'0'  ) THEN
				RDXFER		<=	'1';
				NEXT_STATE	<=	S2;
				STATE_XFER	<=	"10";
			ELSE
				RDXFER		<=	'0';
				NEXT_STATE	<=	S1;
				STATE_XFER	<=	"01";
			END IF;
--===================================================================================================
		WHEN S2	=>							-- STATE 2, ENABLE THE MEMSTOR TRANSFER TO DANTE TX FIFO
			IF ( I2SRD_ADDR > CONV_STD_LOGIC_VECTOR(31,6) ) THEN
				RDXFER		<=	'0';
				NEXT_STATE	<=	S0;
				STATE_XFER	<=	"00";
			ELSE
				RDXFER		<=	'1';
				NEXT_STATE	<=	S2;
				STATE_XFER	<=	"10";
			END IF;
--===================================================================================================
--===================================================================================================
	END CASE;
END PROCESS TRANSITIONS;

END	BEHAVE;
