--===================================================================================================
--	Company			:	Sierra Automated Systems
--	Engineer			:	Anthony Medina
--
--	Create Date		:	06:11:23 02/25/2010 
--	Design Name		:	KRL-Dante
--	Module Name		:	BPI2S_XFERMEM_WRITE - BEHAVE 
--	Project Name		:	KDANTE
--	Target Devices	:	Spartan-3AN
--	Tool versions		:	ISE 9.2
--	Description		:	Simple transfer interface to existing KRL in 32KD frame and
--							:	Dante plug-in module.
--							:
--	Dependencies	:
--
--	Revision			:
--	Revision 0.01 - File Created
--	Additional Comments: 
--
--===================================================================================================
LIBRARY	IEEE;
USE	IEEE.STD_LOGIC_1164.ALL;
USE	IEEE.STD_LOGIC_ARITH.ALL;
USE	IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY	BPI2S_XFERMEM_WRITE is
	PORT(
				BPI2SHOLD	:IN	STD_LOGIC;
				BPI2SCHNL	:IN	STD_LOGIC_VECTOR	(4 DOWNTO 0);
				BPI2SRSYNC	:IN	STD_LOGIC;
				BPMCLK		:IN	STD_LOGIC;
				RST				:IN	STD_LOGIC;
				BPI2S_SEL	:OUT	STD_LOGIC_VECTOR	(3 DOWNTO 0);
				BPI2S_WR		:OUT	STD_LOGIC;
				BPI2S_ADDR	:OUT	STD_LOGIC_VECTOR	(4 DOWNTO 0);
				STATE			:OUT	STD_LOGIC_VECTOR	(2 DOWNTO 0);
				TRANSFER		:OUT	STD_LOGIC
			);
END	BPI2S_XFERMEM_WRITE;

ARCHITECTURE	BEHAVE of BPI2S_XFERMEM_WRITE is

TYPE state_type is (S0, S1, S2, S3, S4, S5, S6);

SIGNAL CURRENT_STATE, NEXT_STATE			:	state_type;										--	STATE transitions
SIGNAL STATE_XFER										:	STD_LOGIC_VECTOR (2 DOWNTO 0);	--	Reflect current state
SIGNAL I2SMEMADDR									:	STD_LOGIC_VECTOR (5 DOWNTO 0);	--	I2S MEM WR ADDR
SIGNAL I2SMUXSEL										:	STD_LOGIC_VECTOR	(3 DOWNTO 0);	--	I2S CHANNEL MUX SELECT
--SIGNAL	WRXFER, MWR, ENAXFER, L_R			:	STD_LOGIC;										--	RD/WR TRANSFER CONTROL SIGNALS
SIGNAL	WRXFER, MWR, ENAXFER					:	STD_LOGIC;										--	RD/WR TRANSFER CONTROL SIGNALS

BEGIN
--===================================================================================================
-- CLOCKED PROCESS
--===================================================================================================
--REGISTERS: PROCESS(BPMCLK, RST, ENAXFER, WRXFER, L_R)				--
REGISTERS: PROCESS(BPMCLK, RST, ENAXFER, WRXFER)						--
	VARIABLE	ADDRCNT	: INTEGER	RANGE 64 DOWNTO 0	:=	0;			--	COUNTER FOR MEM WRITE ADDRESS
	VARIABLE	MUXCNT	: INTEGER	RANGE 32 DOWNTO 0	:=	0;			--	COUNTER FOR CHANNEL MUX SELECT
BEGIN																									--
																											--
	IF RST = '1' THEN																				--
		CURRENT_STATE	<=		S0;														--	Async reset to RSYNC state
		STATE					<=	"000";														--	STATE STARTS AT ZERO
		ADDRCNT				:=		0;															--	SET MEM WRITE ADDRESS COUNT TO ZERO	
		MUXCNT				:=		0;															--	SET I2S CHANNEL MUX SELECT COUNT TO ZERO	
	ELSIF RISING_EDGE(BPMCLK) THEN													--
		CURRENT_STATE	<=	NEXT_STATE;											--
		STATE					<=	STATE_XFER;											--	Output reflects state
			IF (ENAXFER	=	'1') THEN															--	EITHER RD OR WR TRANSFER IS ENABLED
--				IF(L_R	=	'1')THEN																--	SETS THE ADDR COUNTER TO START AT 1
--				ADDRCNT	:=	1;																	--	FOR THE LEFT AUDIO CHANNELS.
--				END IF;																					--
				IF(WRXFER	=	'1')THEN														--
					MWR	<=	NOT MWR;															--
					IF(MWR	=	'1')THEN															--
--						ADDRCNT	:=	ADDRCNT	+	2;										--
						ADDRCNT	:=	ADDRCNT	+	1;										--
						MUXCNT	:=	MUXCNT	+	1;										--
					ELSE																					--
						ADDRCNT	:=	ADDRCNT;											--
						MUXCNT	:=	MUXCNT;												--
					END IF;																				--
				ELSE																						--
					MWR			<=	'0';															--
					ADDRCNT	:=	ADDRCNT;												--
					MUXCNT	:=	MUXCNT;													--
				END IF;																					--
			ELSE																							-- 
				ADDRCNT	:=	0;																	--
				MUXCNT	:=	0;																	--
				MWR			<=	'0';																--
			END IF;																						--
																											--
		I2SMEMADDR		<=	CONV_STD_LOGIC_VECTOR(ADDRCNT,6);	--	
		I2SMUXSEL			<=	CONV_STD_LOGIC_VECTOR(MUXCNT,4);	--	
--	Physical outputs																					--
		BPI2S_SEL(0)		<=	I2SMUXSEL(0);											--	
		BPI2S_SEL(1)		<=	I2SMUXSEL(1);											--	
		BPI2S_SEL(2)		<=	I2SMUXSEL(2);											--	
		BPI2S_SEL(3)		<=	I2SMUXSEL(3);											--	
		BPI2S_WR				<=	MWR;														--	
		BPI2S_ADDR(0)		<=	I2SMEMADDR(0);										--	
		BPI2S_ADDR(1)		<=	I2SMEMADDR(1);										--	
		BPI2S_ADDR(2)		<=	I2SMEMADDR(2);										--	
		BPI2S_ADDR(3)		<=	I2SMEMADDR(3);										--	
		BPI2S_ADDR(4)		<=	I2SMEMADDR(4);										--
		TRANSFER				<=	ENAXFER;													--
	END IF;

END PROCESS REGISTERS;
--===================================================================================================
-- ASYNC PROCESS
--===================================================================================================
TRANSITIONS: PROCESS(CURRENT_STATE, BPI2SRSYNC, I2SMEMADDR, BPI2SHOLD, BPI2SCHNL)
BEGIN
	CASE CURRENT_STATE is
--===================================================================================================
--===================================================================================================
		WHEN S0	=>							-- STATE 0, INITIAL START CHECKS FOR BACKPLANE RESYNC
			IF ( BPI2SRSYNC	=	'1' ) THEN
				ENAXFER		<=	'0';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S1;
				STATE_XFER	<=	"001";
			ELSE
				ENAXFER		<=	'0';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S0;
				STATE_XFER	<=	"000";
			END IF;
--===================================================================================================
		WHEN S1	=>							-- STATE 1, CHECK TO SEE IF THE LOAD SIGNAL AND I2S CHANNEL COUNT MATCH TO KNOW WHERE ZERO IS
			IF ( BPI2SHOLD	=	'1'	AND	BPI2SCHNL = CONV_STD_LOGIC_VECTOR(0,4) ) THEN
				ENAXFER		<=	'0';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S2;
				STATE_XFER	<=	"010";
			ELSE
				ENAXFER		<=	'0';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S1;
				STATE_XFER	<=	"001";
			END IF;
--===================================================================================================
		WHEN S2	=>							-- STATE 2, WAIT FOR THE BPI2S HOLD TO GO LO THEN TRANSFER THE LEFT I2S AUDIO CHANNELS TO MEM
			IF ( BPI2SHOLD	=	'0' ) THEN
				ENAXFER		<=	'0';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S3;
				STATE_XFER	<=	"011";
			ELSE
				ENAXFER		<=	'0';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S2;
				STATE_XFER	<=	"010";
			END IF;
--===================================================================================================
		WHEN S3	=>							-- STATE 3, TRANSFER THE LEFT 16 (I2S) CHANNELS TO MEMORY
--			IF ( I2SMEMADDR > CONV_STD_LOGIC_VECTOR(30,6) ) THEN
			IF ( I2SMEMADDR > CONV_STD_LOGIC_VECTOR(15,6) ) THEN
				ENAXFER		<=	'1';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S4;
				STATE_XFER	<=	"100";
			ELSE
				ENAXFER		<=	'1';
				WRXFER		<=	'1';
--				L_R				<=	'0';
				NEXT_STATE	<=	S3;
				STATE_XFER	<=	"011";
			END IF;
--===================================================================================================
		WHEN S4	=>							-- STATE 4, WAIT FOR THE NEXT BPI2S HOLD TO GO HI
			IF ( BPI2SHOLD	=	'1' ) THEN
				ENAXFER		<=	'1';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S5;
				STATE_XFER	<=	"101";
			ELSE
				ENAXFER		<=	'1';
				WRXFER		<=	'0';
--				L_R				<=	'1';
				NEXT_STATE	<=	S4;
				STATE_XFER	<=	"100";
			END IF;
--===================================================================================================
		WHEN S5	=>							-- STATE 5, WAIT FOR THE NEXT BPI2S HOLD TO GO LO
			IF ( BPI2SHOLD	=	'0' ) THEN
				ENAXFER		<=	'1';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S6;
				STATE_XFER	<=	"110";
			ELSE
				ENAXFER		<=	'1';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S5;
				STATE_XFER	<=	"101";
			END IF;
--===================================================================================================
		WHEN S6	=>							-- STATE 6, TRANSFER THE RIGHT 16 (I2S) CHANNELS TO MEMORY
			IF ( I2SMEMADDR > CONV_STD_LOGIC_VECTOR(31,6) ) THEN
				ENAXFER		<=	'0';
				WRXFER		<=	'0';
--				L_R				<=	'0';
				NEXT_STATE	<=	S1;
				STATE_XFER	<=	"001";
			ELSE
				ENAXFER		<=	'1';
				WRXFER		<=	'1';
--				L_R				<=	'0';
				NEXT_STATE	<=	S6;
				STATE_XFER	<=	"110";
			END IF;
--===================================================================================================
--===================================================================================================
	END CASE;
END PROCESS TRANSITIONS;

END	BEHAVE;

