--===================================================================================================
--	Company			:	Sierra Automated Systems
--	Engineer			:	Anthony Medina
-- 						:
--	Create Date		:	11:57:45 02/21/2010 
--	Design Name		:	KRL Dante
--	Module Name		:	DANTE_RXMEM_TO_BPMEM - BEHAVE 
--	Project Name		:	KRL Link to Dante Conversion
--	Target Devices	:	Spartan-3AN
--	Tool versions		:	ISE 9.2
--	Description		:
--							:
--	Dependencies	:
--							:
--	Revision			:
--	Revision 0.01 - File Created
--	Additional Comments: 
--
--===================================================================================================
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY	DANTE_RXMEM_TO_BPMEM is
	PORT(
				FIFO_XFR_DONE	:IN	STD_LOGIC;
				BP_CLK					:IN	STD_LOGIC;
				RST						:IN	STD_LOGIC;
				MEM_RDADDR		:OUT	STD_LOGIC_VECTOR (4 DOWNTO 0);
				MEM_RD				:OUT	STD_LOGIC;
				MEM_WRADDR		:OUT	STD_LOGIC_VECTOR (4 DOWNTO 0);
				MEM_WR				:OUT	STD_LOGIC;
				STATE					:OUT	STD_LOGIC_VECTOR	(2 DOWNTO 0)
			);
END	DANTE_RXMEM_TO_BPMEM;

ARCHITECTURE	BEHAVE of DANTE_RXMEM_TO_BPMEM is

TYPE state_type is (S0, S1, S2, S3);

SIGNAL CURRENT_STATE, NEXT_STATE		:	state_type;
SIGNAL STATE_XFER									:	STD_LOGIC_VECTOR (2 DOWNTO 0);
SIGNAL TRANSFER										:	STD_LOGIC;
SIGNAL READ_ADDR, WRITE_ADDR			:	STD_LOGIC_VECTOR (5 DOWNTO 0);

BEGIN
--===================================================================================================
-- CLOCKED PROCESS
--===================================================================================================
REGISTERS: PROCESS(BP_CLK, RST, STATE_XFER, FIFO_XFR_DONE)
	VARIABLE	RDADDR	: INTEGER	RANGE 63 DOWNTO 0	:=	0;	-- 6-bit counter to allow for count above 31, only 5-bits used for ADDR
	VARIABLE	WRADDR	: INTEGER	RANGE 63 DOWNTO 0	:=	0;	-- 6-bit counter to allow for count above 31, only 5-bits used for ADDR

BEGIN
	IF RST = '1' THEN
		MEM_RDADDR		<=	"00000";			-- Read address for RXFIFO memstor
		MEM_WRADDR		<=	"00000";			-- Write address for BPLVDS transfer mem
		MEM_RD				<=		'0';				-- RD enable for RXFIFO memstor
		MEM_WR				<=		'0';				-- WR enable for BPLVDS transfer mem
		STATE					<=	"000";				-- State output to zero
		CURRENT_STATE	<=		S0;				-- Start at state 0
		TRANSFER				<=		'0';				-- Monitors RXFIFO transfer, HI to LO starts XFER from RXFIFO mem to BPLVDS mem
		RDADDR				:=		0;					-- Set RDADDR to zero
		WRADDR				:=		0;					-- Set WRADDR to zero	
		READ_ADDR			<=	"000000";			--
		WRITE_ADDR		<=	"000000";			--
	ELSIF RISING_EDGE(BP_CLK) THEN			--
		CASE CURRENT_STATE is
--===================================================================================================
--===================================================================================================
		WHEN S0	=>										-- STATE 0
		TRANSFER		<=	'0';							--
		MEM_RD		<=	'0';							--
		MEM_WR		<=	'0';							--
		RDADDR		:=	0;								--
		WRADDR		:=	0;								--
			IF ( FIFO_XFR_DONE	=	'1' ) THEN		--
				NEXT_STATE	<=	S1;					--
				STATE_XFER	<=	"001";				--
			ELSE													--
				NEXT_STATE	<=	S0;					--
				STATE_XFER	<=	"000";				--
			END IF;												--
--===================================================================================================
		WHEN S1	=>										-- STATE 1
		TRANSFER		<=	'0';							--
		MEM_RD		<=	'0';							--
		MEM_WR		<=	'0';							--
		RDADDR		:=	0;								--
		WRADDR		:=	0;								--
			IF ( FIFO_XFR_DONE	=	'0' ) THEN		--
				MEM_RD		<=	'1';					--
				TRANSFER		<=	'1';					--
				NEXT_STATE	<=	S2;					--
				STATE_XFER	<=	"010";				--
			ELSE													--
				NEXT_STATE	<=	S1;					--
				STATE_XFER	<=	"001";				--
			END IF;												--
--===================================================================================================
		WHEN S2	=>										-- STATE 2
		TRANSFER		<=	TRANSFER;				--
		MEM_RD		<=	'0';							--
		MEM_WR		<=	'0';							--
		RDADDR		:=	RDADDR;					--
		WRADDR		:=	WRADDR;					--
			IF ( TRANSFER	=	'0' ) THEN				--
				RDADDR		:=	0;						--
				WRADDR		:=	0;						--
				NEXT_STATE	<=	S0;					--
				STATE_XFER	<=	"000";				--
			ELSE													--
				IF ( RDADDR	= 30 ) THEN				--
					RDADDR	:=	1;						--
				ELSIF(RDADDR	=	31)THEN			--
					RDADDR	:=	RDADDR;			--
				ELSE												--
					RDADDR	:=	RDADDR + 2;	--
				END IF;											--
				MEM_WR	<=	'1';						--
				NEXT_STATE	<=	S3;					--
				STATE_XFER	<=	"011";				--
			END IF;												--
--===================================================================================================
		WHEN S3	=>										-- STATE 3
		TRANSFER		<=	TRANSFER;				--
		MEM_RD		<=	'0';							--
		MEM_WR		<=	'0';							--
		RDADDR		:=	RDADDR;					--
		WRADDR		:=	WRADDR;					--
			IF ( WRADDR	= 31 ) THEN					--
				TRANSFER		<=	'0';					--
				WRADDR		:=	0;						--
				RDADDR		:=	0;						--
				MEM_RD		<=	'0';					--
				NEXT_STATE	<=	S0;					--
				STATE_XFER	<=	"000";				--
			ELSE													--
				IF(WRADDR	>	30 )THEN				--
					MEM_RD	<=	'0';					--
				ELSE												--
					MEM_RD	<=	'1';					--
				END IF;											--
				WRADDR		:=	WRADDR + 1;	--
				NEXT_STATE	<=	S2;					--
				STATE_XFER	<=	"010";				--
			END IF;												--
--===================================================================================================
--===================================================================================================
	END CASE;

		READ_ADDR			<=	CONV_STD_LOGIC_VECTOR(RDADDR,6);
		WRITE_ADDR		<=	CONV_STD_LOGIC_VECTOR(WRADDR,6);

	END IF;

		CURRENT_STATE	<=	NEXT_STATE;		--
		STATE					<=	STATE_XFER;		-- Output reflects state
-- READ ADDRESS												--
		MEM_RDADDR(0)	<=	READ_ADDR(0);	-- 
		MEM_RDADDR(1)	<=	READ_ADDR(1);	-- 
		MEM_RDADDR(2)	<=	READ_ADDR(2);	--	Dante RX FIFO memstor read address
		MEM_RDADDR(3)	<=	READ_ADDR(3);	-- 
		MEM_RDADDR(4)	<=	READ_ADDR(4);	-- 
--	WRITE ADDRESS											--
		MEM_WRADDR(0)	<=	WRITE_ADDR(0);	-- 
		MEM_WRADDR(1)	<=	WRITE_ADDR(1);	-- 
		MEM_WRADDR(2)	<=	WRITE_ADDR(2);	--	Dante to BackPlane XFER mem write address
		MEM_WRADDR(3)	<=	WRITE_ADDR(3);	-- 
		MEM_WRADDR(4)	<=	WRITE_ADDR(4);	-- 

END PROCESS REGISTERS;

END	BEHAVE;

