--===================================================================================================
-- Company: 		SIERRA AUTOMATED SYSTEMS
-- Engineer:			ANTHONY MEDINA
-- 
-- Create Date:		12:29:24 03/01/2010 
-- Design Name:	DANTE TX HOLD REGISTER WRITE LOGIC
-- Module Name:	DANTE_TX_HOLD_WRITE - BEHAVE 
-- Project Name:	SUPER RIO
-- Target Devices:	SPARTAN-3ADSP
-- Tool versions:	ISE
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
--===================================================================================================
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY DANTE_TX_HOLD_WRITE is
	PORT(
				D_TXFIFO_CHNCNT	:IN	STD_LOGIC_VECTOR (12 DOWNTO 0);
				D_TXFRAME_DELAY	:IN	STD_LOGIC_VECTOR	(12 DOWNTO 0);
				D_RDRSYNC				:IN	STD_LOGIC;
				D_CHNCNT				:IN	STD_LOGIC_VECTOR	(3 DOWNTO 0);
				D_CHN_LD				:IN	STD_LOGIC;
				BP_CLK						:IN	STD_LOGIC;
				RST							:IN	STD_LOGIC;
				STATE						:OUT	STD_LOGIC_VECTOR	(2 DOWNTO 0);
				D_TXLD_ENA				:OUT	STD_LOGIC
			);
END DANTE_TX_HOLD_WRITE;

ARCHITECTURE BEHAVE of DANTE_TX_HOLD_WRITE is

TYPE state_type is (S0, S1, S2, S3, S4);

SIGNAL CURRENT_STATE, NEXT_STATE		:	state_type;
SIGNAL STATE_XFER									:	STD_LOGIC_VECTOR (2 downto 0);
SIGNAL ENA_XFER, D_CHNL_LOAD				:	STD_LOGIC;
SIGNAL PULSECOUNT									:	STD_LOGIC_VECTOR (3 downto 0);

BEGIN
--===================================================================================================
-- CLOCKED PROCESS
--===================================================================================================
REGISTERS: PROCESS(BP_CLK, RST, STATE_XFER, ENA_XFER)
	VARIABLE	TCNT	: INTEGER	RANGE 15 DOWNTO 0	:=	0;	-- 4-bit counter
BEGIN
	IF RST = '1' THEN
		CURRENT_STATE	<=	S0;												-- Async reset to RSYNC state
		STATE					<=	"000";											-- State output to zero
		TCNT					:=	0;													-- Set pulse count to zero	
		D_CHNL_LOAD		<=	'0';												--	
	ELSIF RISING_EDGE(BP_CLK) THEN										--
		CURRENT_STATE	<=	NEXT_STATE;								--
		STATE					<=	STATE_XFER;								-- Output reflects state
		D_CHNL_LOAD		<=	D_CHN_LD;
			IF (ENA_XFER	=	'1') THEN											--
				TCNT 	:=	TCNT + 1;												-- Inc the TX enable counter
			ELSE																				-- 
				TCNT 	:=	0;															-- Else keep tcnt zero  
			END IF;																			--
		PULSECOUNT		<= CONV_STD_LOGIC_VECTOR(TCNT,4);	-- Async combinatorial section uses this to shut off the transfers
		D_TXLD_ENA		<=	ENA_XFER;										-- Dante RX FIFO write enable
	END IF;
END PROCESS REGISTERS;

--===================================================================================================
-- ASYNC PROCESS
--===================================================================================================
TRANSITIONS: PROCESS(CURRENT_STATE, PULSECOUNT, D_CHNCNT, D_CHNL_LOAD, D_TXFIFO_CHNCNT, D_TXFRAME_DELAY)
BEGIN
	CASE CURRENT_STATE is
--===================================================================================================
--===================================================================================================
		WHEN S0	=>						-- STATE 0
			IF ( D_RDRSYNC	=	'1' AND D_CHNCNT =	CONV_STD_LOGIC_VECTOR(0,4) AND (D_TXFIFO_CHNCNT	>=	D_TXFRAME_DELAY) ) THEN
				ENA_XFER		<=		'0';
				NEXT_STATE	<=		S1;
				STATE_XFER	<=	"010";
			ELSE
				ENA_XFER		<=		'0';
				NEXT_STATE	<=		S0;
				STATE_XFER	<=	"000";
			END IF;
--===================================================================================================
		WHEN S1	=>						-- STATE 1
			IF ( D_CHNL_LOAD	=	'1'	AND	D_CHNCNT	=	CONV_STD_LOGIC_VECTOR(0,4)) THEN
				ENA_XFER		<=		'0';
				NEXT_STATE	<=		S3;
				STATE_XFER	<=	"011";
			ELSE
				ENA_XFER		<=		'0';
				NEXT_STATE	<=		S1;
				STATE_XFER	<=	"001";
			END IF;
--===================================================================================================
		WHEN S2	=>						-- STATE 2
			IF ( D_CHNL_LOAD	=	'1' ) THEN
				ENA_XFER		<=		'0';
				NEXT_STATE	<=		S3;
				STATE_XFER	<=	"011";
			ELSE
				ENA_XFER		<=		'0';
				NEXT_STATE	<=		S2;
				STATE_XFER	<=	"010";
			END IF;
--===================================================================================================
		WHEN S3	=>						-- STATE 3
			IF ( D_CHNL_LOAD	=	'0' ) THEN
				ENA_XFER		<=		'0';
				NEXT_STATE	<=		S4;
				STATE_XFER	<=	"100";
			ELSE
				ENA_XFER		<=		'0';
				NEXT_STATE	<=		S3;
				STATE_XFER	<=	"011";
			END IF;
--===================================================================================================
		WHEN S4	=>						-- STATE 4
			IF ( PULSECOUNT > CONV_STD_LOGIC_VECTOR(7,4)) THEN
				ENA_XFER		<=		'0';
				NEXT_STATE	<=		S2;
				STATE_XFER	<=	"010";
			ELSE
				ENA_XFER		<=		'1';
				NEXT_STATE	<=		S4;
				STATE_XFER	<=	"100";
			END IF;
--===================================================================================================
--===================================================================================================
	END CASE;
END PROCESS TRANSITIONS;

END BEHAVE;
