--=========================================================================================================
--
--	Jitter to Wander
--	Anthony Medina
--	Sierra Automated Systems
--	Jan 16, 2017
--
--	Attempt to minimize the LRCLK Jitter from ANI Recovered clock down to more of a "Wander" so that
--	the Brooklyn-II Module will lock on and stay locked. Not sure how thiw will affect the SCLK as the
--	Falling-Falling edge phase relationship may be offset and cuase bit shifting.
--
--=========================================================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity clock_stabilizer is
	generic(
			integer_part	: integer := 12;
			fractional_part	: integer := 10
			);
	port(
		clk_100			: in std_logic;
		wordclock_in	: in std_logic;
		wordclock_out	: out std_logic
		);
end entity clock_stabilizer;

architecture Behave of clock_stabilizer is

	signal wordclock_shift					: std_logic_vector(1 downto 0);
	signal wordclock_in_counter				: std_logic_vector(fractional_part-1 downto 0);
	signal wordclock_time_add				: std_logic_vector(integer_part+fractional_part-1 downto 0);
	signal wordclock_time_total				: std_logic_vector(integer_part+fractional_part-1 downto 0);
	signal wordclock_period_int_counter		: std_logic_vector(integer_part-1 downto 0);
	signal wordclock_period_frac_counter	: std_logic_vector(fractional_part-1 downto 0);

begin

wordclock_in_processing : process (clk_100)
begin
	if rising_edge(clk_100) then
		wordclock_shift <= wordclock_shift(0) & wordclock_in;
			if wordclock_shift = "01" then
				if wordclock_in_counter = 2**wordclock_in_counter'length-1 then
					wordclock_time_total	<=	wordclock_time_add;
					wordclock_time_add		<=	(others => '0');
					wordclock_in_counter	<=	(others => '0');
				else
					wordclock_in_counter <= wordclock_in_counter + 1;
				end if;
			else
				wordclock_time_add <= wordclock_time_add + 1;
			end if;
	end if;
end process wordclock_in_processing;
 
wordclock_out_processing : process (clk_100)
begin
  if rising_edge(clk_100) then
   if wordclock_shift = "01" and wordclock_in_counter = 2**wordclock_in_counter'length-1 then
    wordclock_period_frac_counter <= (others => '0');
    wordclock_period_int_counter <= (others => '0');
    wordclock_out <= '1';
   else
    if wordclock_period_frac_counter <= wordclock_time_total(fractional_part-1 downto 0) + 1 then
     if wordclock_period_int_counter <= wordclock_time_total(integer_part+fractional_part-1 downto fractional_part) then
      wordclock_period_int_counter <= wordclock_period_int_counter + 1;
      if wordclock_period_int_counter = '0' & wordclock_time_total(integer_part+fractional_part-1 downto fractional_part+1) then
       wordclock_out <= '0';
      end if; 
     else
      wordclock_period_int_counter <= (others => '0');
      wordclock_out <= '1';
      wordclock_period_frac_counter <= wordclock_period_frac_counter + 1;
     end if;
    else
     if wordclock_period_int_counter < wordclock_time_total(integer_part+fractional_part-1 downto fractional_part) then
      wordclock_period_int_counter <= wordclock_period_int_counter + 1;
      if wordclock_period_int_counter = '0' & wordclock_time_total(integer_part+fractional_part-1 downto fractional_part+1) then
       wordclock_out <= '0';
      end if; 
     else
      wordclock_period_int_counter <= (others => '0');
      wordclock_out <= '1';
      wordclock_period_frac_counter <= wordclock_period_frac_counter + 1;
     end if;  
    end if;
   end if;
  end if;
 end process wordclock_out_processing;

end Behave;
