-- Generic Terminal counter
-- Dec 22, 2004
-- Anthony Medina
-- Made this a sync reset
--
LIBRARY IEEE ;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY WDCLKSYNC_TCNT4SEC IS
	GENERIC(WIDTH	: INTEGER	:= 24);
PORT(
		CLK		:IN		STD_LOGIC;
		RST		:IN		STD_LOGIC;
		TCNT	:OUT	STD_LOGIC
	);
END WDCLKSYNC_TCNT4SEC;

ARCHITECTURE Behave OF WDCLKSYNC_TCNT4SEC IS

SIGNAL	QTMP		:	STD_LOGIC_VECTOR (WIDTH-1 DOWNTO 0);
SIGNAL	ENDCOUNT	:	STD_LOGIC;

BEGIN
PROCESS(CLK,RST,QTMP)
--
--	Use 3 for simulation.
--	1,440,000	Dec	=>	15F900	Hex:	30	seconds,	16#15F900#
--	960,000		Dec	=>	0EA600	Hex:	20	seconds,	16#0EA600#
--	480,000		Dec	=>	075300	Hex:	10	seconds,	16#075300#
--	144,000		Dec	=>	23280	Hex:	3	seconds,	16#23280#
--	 48,000		Dec	=>	BB80	Hex:	1	second,		16#BB80#
--
--	Try to keep this as minimal as possible but enough to allow the Dante
--	to boot up and switch over SCLK from base 3MHz to 24MHz.
--	This takes approximately 3 to 7 seconds
--
--	CONSTANT	TIMER			:	INTEGER	:=	16#15F900#;	--	30	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#0EA600#;	--	DEFAULT 20 seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#075300#;	--	10	seconds
	CONSTANT	TIMER			:	INTEGER	:=	16#2EE00#;	--	4	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#23280#;	--	3	seconds
--	CONSTANT	TIMER			:	INTEGER	:=	16#BB80#;	--	1	second
--	CONSTANT	TIMER			:	INTEGER	:=	16#8340#;	--	700 mS
--	CONSTANT	TIMER			:	INTEGER	:=	16#5A00#;	--	480 mS
--	CONSTANT	TIMER			:	INTEGER	:=	16#000003#;	--	Use this for simulation only
--
--
BEGIN

IF(RST = '1') THEN
	QTMP		<=	CONV_STD_LOGIC_VECTOR(0,WIDTH);
	ENDCOUNT	<=	'1';
ELSIF rising_edge(CLK) THEN
	IF(QTMP	< CONV_STD_LOGIC_VECTOR(TIMER,WIDTH)) THEN
		QTMP		<=	QTMP + '1';
		ENDCOUNT	<=	'1';
	ELSE
		QTMP		<=	CONV_STD_LOGIC_VECTOR(TIMER,WIDTH);
		ENDCOUNT	<=	'0';
	END IF;
END IF;

END PROCESS;

	TCNT	<=	ENDCOUNT;

END Behave;
