-- Generic Terminal counter
-- Dec 22, 2004
-- Anthony Medina
-- Made this a sync reset
--
LIBRARY IEEE ;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY WRDCLKWINDOW IS
	GENERIC(WIDTH	: INTEGER	:=	10);
PORT(
		CLK			:IN		STD_LOGIC;
		RST			:IN		STD_LOGIC;
		CHKWINDOW	:OUT	STD_LOGIC
	);
END WRDCLKWINDOW;

ARCHITECTURE Behave OF WRDCLKWINDOW IS

SIGNAL	QTMP		:	STD_LOGIC_VECTOR (WIDTH-1 DOWNTO 0);
SIGNAL	ENDCOUNT	:	STD_LOGIC;

BEGIN

COUNTER:
PROCESS(CLK,RST,QTMP)
BEGIN
	IF(RST = '1') THEN
		QTMP	<=	CONV_STD_LOGIC_VECTOR(0,WIDTH);
	ELSIF rising_edge(CLK) THEN
		QTMP	<=	QTMP + '1';
	END IF;
END PROCESS COUNTER;

--	0x3D7 => 983 / 0x28 => 40	initial window size
--	0x3CD => 973 / 0x32 => 50	expanded window size
--
--
--

WINDOW:
PROCESS(QTMP,RST)
BEGIN
	IF(RST = '1') THEN
		ENDCOUNT	<=	'1';
	ELSIF(	(QTMP >= CONV_STD_LOGIC_VECTOR(16#3CD#,WIDTH) AND QTMP <= CONV_STD_LOGIC_VECTOR(16#3FF#,WIDTH)) OR
			(QTMP >= CONV_STD_LOGIC_VECTOR(0,WIDTH) AND QTMP <= CONV_STD_LOGIC_VECTOR(16#32#,WIDTH)) ) THEN
		ENDCOUNT	<=	'1';
	ELSE
		ENDCOUNT	<=	'0';
	END IF;
	CHKWINDOW	<=	ENDCOUNT;
END PROCESS WINDOW;

END Behave;
